import 'dart:io';
import 'package:get_cli/models/file_model.dart';
import 'package:get_cli/sample_interface.dart';
import 'package:get_cli/structure.dart';
import 'package:recase/recase.dart';
import 'functions/create.dart';
import 'functions/create_init.dart';

/// This is an defaultGenerator which takes [defaultStructure] as an parameter.
Generator defaultGenerator = Generator(currentStructure: defaultStructure);

/// Generator takes an [Structure] object, to specify where to create .dart files
class Generator {
  final Structure currentStructure;

  Generator({
    this.currentStructure,
  }) : assert(currentStructure != null);

  /// This function only handles file creation
  Future<File> createFile({FileModel fileModel, Sample content}) async {
    var reCase = ReCase(fileModel.result);
    var _file = await File(fileModel.path + '.dart').create(recursive: true);
    await _file.writeAsString(content.file(reCase.pascalCase));

    print(
        'file created succesfully with name :${fileModel.result} at path: ${fileModel.path}');
    return _file;
  }
}

/// Essa função é chamada pela main, e recebe os argumentos da cli
Future<void> generate({
  List<String> arguments,
}) async {
  print("list de arguments = $arguments");

  final validate = validateArgs(arguments);
  if (!validate) {
    print('Error!!!!!!!!!!!! wrong arguments');
  }

  switch (arguments.first) {
    case "init":
      await createInitial();
      break;
    case "upgrade":
      //TODO insert upgrade funcion
      break;
    case "install":
      //TODO insert install funcion
      break;
    case "remove":
      //TODO insert remove funcion
      break;
    case "create":
      await create(arguments);
      break;
  }
}

bool validateArgs(List<String> arguments) {
  List<String> firstArgsAllow = [
    'create',
    'init',
    'upgrade',
    'install',
    'remove'
  ];
  List<String> secondArgsAllow = [
    'page',
    'controller',
    'route',
    'presentation',
    'view'
  ];
  if (arguments != null &&
      arguments.isNotEmpty &&
      firstArgsAllow.contains(arguments.first)) {
    if (arguments.first == 'init') return true;
    final secondArg = arguments.first.split(':').last;
    if (secondArgsAllow.contains(secondArg)) return true;
  }
  return false;
}
